<?php
// BinaryWebSocketClient.php

declare(strict_types=1);

class BinaryWebSocketClient
{
    private \WebSocket\Client $socket;
    private array $handlers = [];
    private bool $isRunning = false;

    public function __construct(
        string $url,
        private array $options = [
            'timeout' => 5,
            'heartbeat_interval' => 30
        ]
    ) {
        $this->socket = new \WebSocket\Client($url, [
            'timeout' => $this->options['timeout']
        ]);
    }

    /**
     * 注册消息处理器
     */
    public function on(int $msgId, callable $handler): void
    {
        $this->handlers[$msgId] = $handler;
    }

    /**
     * 发送二进制消息
     * @param int $msgId 使用MsgID常量
     * @param \Google\Protobuf\Internal\Message $data Protobuf消息对象
     */
    public function sendBinary(int $msgId, \Google\Protobuf\Internal\Message $data): void
    {
        $binMsgId = pack('n', $msgId); // 2字节无符号短整型
        $binData = $binMsgId . $data->serializeToString();
        $this->socket->send($binData, \WebSocket\Client::BINARY);
    }

    /**
     * 启动消息监听循环
     */
    public function startListening(): void
    {
        if ($this->isRunning) {
            return;
        }

        $this->isRunning = true;
        $this->startHeartbeat();

        while ($this->isRunning) {
            try {
                $message = $this->socket->receive();
                $this->dispatchBinaryMessage($message);
            } catch (\WebSocket\ConnectionException $e) {
                $this->handleDisconnect($e);
            }
        }
    }

    /**
     * 解析二进制消息
     */
    private function dispatchBinaryMessage(string $binary): void
    {
        if (strlen($binary) < 2) {
            throw new \RuntimeException("Invalid binary message length");
        }

        // 提取MsgID (前2字节)
        $msgId = unpack('n', substr($binary, 0, 2))[1];
        $protoData = substr($binary, 2);

        if (isset($this->handlers[$msgId])) {
            try {
                // 根据MsgID获取对应的Protobuf类
                $protoClass = $this->getProtoClassByMsgId($msgId);
                $message = new $protoClass();
                $message->mergeFromString($protoData);
                $this->handlers[$msgId]($message);
            } catch (\Exception $e) {
                error_log("Decode error for MsgID {$msgId}: " . $e->getMessage());
            }
        }
    }

    /**
     * MsgID到Protobuf类的映射
     */
    private function getProtoClassByMsgId(int $msgId): string
    {
        static $map = [
            MsgID::REQLOGIN => \Protocol\ReqLogin::class,
            MsgID::RESLOGIN => \Protocol\ResLogin::class,
            // 其他MsgID映射...
        ];

        if (!isset($map[$msgId])) {
            throw new \RuntimeException("Unregistered MsgID: {$msgId}");
        }

        return $map[$msgId];
    }

    private function startHeartbeat(): void
    {
        if ($this->options['heartbeat_interval'] <= 0) {
            return;
        }

        \React\EventLoop\Loop::addPeriodicTimer(
            $this->options['heartbeat_interval'],
            fn() => $this->sendBinary(MsgID::REQHEARTBEAT, new \Protocol\Heartbeat())
        );
    }

    private function handleDisconnect(\Throwable $e): void
    {
        // 实现带退避的重连
        static $retryCount = 0;
        $delay = min(pow(2, $retryCount), 30); // 指数退避，最大30秒
        
        sleep($delay);
        $this->socket = new \WebSocket\Client(
            $this->socket->getUrl(),
            ['timeout' => $this->options['timeout']]
        );
        
        $retryCount++;
    }

    public function close(): void
    {
        $this->isRunning = false;
        $this->socket->close();
    }
}